package net.sourceforge.rapidprops;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/** Used to configure the global characteristics of the implementation generated for the annotated interface. */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface Props {
	/** Provides a handling method used to adjust a string. */
	public interface Handler {
		String handle(String data);
	}

	/** A handler that does not change the data. */
	public static final class NoHandling implements Handler {
		@Override
		public String handle(String data) {
			return data;
		}
	}

	/** A handler that converts the string to upper case. */
	public static final class ToUpperCase implements Handler {
		@Override
		public String handle(String data) {
			return (data == null) ? null : data.toUpperCase();
		}
	}

	/** A handler that converts the string to lower case. */
	public static final class ToLowerCase implements Handler {
		@Override
		public String handle(String data) {
			return (data == null) ? null : data.toLowerCase();
		}
	}
	
	public static class ToCapitalizedFirstLetter implements Props.Handler {
		@Override
		public String handle(String data) {
			data = data.toLowerCase();
			StringBuilder sb = new StringBuilder();
			boolean shouldCapitalize = true;
			for(int i=0; i<data.length(); i++) {
				char ch = data.charAt(i);
				int codePoint = data.codePointAt(i);
				if(Character.isLetter(codePoint)) {
					if(shouldCapitalize) {
						ch = Character.toUpperCase(ch);
					}
					shouldCapitalize = false;
				} else {
					shouldCapitalize = true;
				}
				sb.append(ch);
			}
			return sb.toString();
		}		
	}
	
	/** If true, existing keys will be trimmed. */
	boolean trimKeys() default true;

	/** If true, existing values will be trimmed. */
	boolean trimValues() default true;
	
	/** The handler used to process existing keys. Frequently used handlers: {@link ToUpperCase}, {@link ToLowerCase} */
	Class<? extends Handler> keyHandler() default NoHandling.class;
	
	/** The handler used to process existing values. */
	Class<? extends Handler> valueHandler() default NoHandling.class;
}
